package gg.malloc.defense.engine;

import gg.malloc.defense.model.Game;
import gg.malloc.defense.model.Progress;
import gg.malloc.defense.model.State;

import org.bukkit.entity.Player;

public class GameState implements State {
  GameRunner m_runner;
  WaveManager m_waves;
  Game m_game;
  MobManager m_mobs;
  BombFuse m_fuse;
  PlayerManager m_players;

  int m_coinsDropped;
  int m_coinsPickedUp;

  public GameState(GameRunner runner, WaveManager waves, Game game, MobManager mobs, BombFuse fuse, PlayerManager players) {
    m_runner = runner;
    m_waves = waves;
    m_game = game;
    m_mobs = mobs;
    m_fuse = fuse;
    m_players = players;
    m_coinsDropped = 0;
    m_coinsPickedUp = 0;
  }

  public int getRemainingCoins() {
    return coinsDropped() - coinsPickedUp();
  }

  public int coinsDropped() {
    return m_coinsDropped;
  }

  public int coinsPickedUp() {
    return m_coinsPickedUp;
  }

  int INITIAL_BALANCE = 1500;

  public int getPlayerBalance(Player p) {
    return coinsPickedUp() - m_players.getPlayerExpenses(p) + INITIAL_BALANCE;
  }

  public void addDroppedCoins(int v) {
    m_coinsDropped += v;
  }

  public void addPickedUpCoins(int v) {
    m_coinsPickedUp += v;
  }

  public void resetBalances() {
    m_coinsDropped = 0;
    m_coinsPickedUp = 0;
  }

  public Progress playerRespawnProgress(Player p) {
    return m_players.getRespawnProgress(p);
  }

  public Progress coinProgress() {
    return new StaticProgress(m_coinsPickedUp, m_coinsDropped);
  }

  public GameRunner.Stage getStage() {
    return m_runner.getStage();
  }

  public Progress fuseProgress() {
    return m_fuse;
  }

  public Progress waveProgress() {
    return m_waves.asProgress();
  }

  public Progress mobProgress() {
    return m_mobs;
  }

  public Progress playerReadyProgress() {
    return m_players.readyProgress();
  }
}
