package gg.malloc.defense.ui;

import gg.malloc.defense.model.Game;
import gg.malloc.defense.model.State;
import gg.malloc.defense.model.Progress;

import java.util.HashMap;

import org.bukkit.boss.BossBar;
import org.bukkit.boss.BarStyle;
import org.bukkit.boss.BarColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

public class BossBars {
  BossBar m_gameBar = Bukkit.createBossBar("Malloc Defense", BarColor.PURPLE, BarStyle.SOLID);
  BossBar m_waveBar = Bukkit.createBossBar("Malloc Defense", BarColor.PURPLE, BarStyle.SOLID);
  BossBar m_bombBar = Bukkit.createBossBar("Bomb Fuse", BarColor.RED, BarStyle.SOLID);
  HashMap<Player, BossBar> m_respawnBars = new HashMap<>();

  State m_gameState;

  double m_countdownProgress;

  public void setCountdownProgress(double v) {
    m_countdownProgress = v;
  }

  public BossBars(State gameState) {
    m_gameState = gameState;
    m_gameBar.setVisible(true);
    m_waveBar.setVisible(false);
    m_bombBar.setVisible(false);
  }

  public void addPlayer(Player p) {
    m_gameBar.addPlayer(p);
    m_waveBar.addPlayer(p);
    m_bombBar.addPlayer(p);
    BossBar respawnBar = Bukkit.createBossBar("Respawn", BarColor.BLUE, BarStyle.SOLID);
    respawnBar.addPlayer(p);
    respawnBar.setVisible(false);
    m_respawnBars.put(p, respawnBar);
  }

  public void removePlayer(Player p) {
    m_gameBar.removePlayer(p);
    m_waveBar.removePlayer(p);
    m_bombBar.removePlayer(p);
    if (m_respawnBars.containsKey(p)) {
      m_respawnBars.get(p).removePlayer(p);
      m_respawnBars.remove(p);
    }
  }

  public void update() {
    switch(m_gameState.getStage()) {
      case Idle:
        m_gameBar.setProgress(1.0);
        m_gameBar.setTitle("Waiting for players...");
        m_gameBar.setColor(BarColor.PURPLE);
        m_waveBar.setVisible(false);
        m_bombBar.setVisible(false);
        break;
      case Warmup:
        m_gameBar.setProgress(m_gameState.waveProgress().toDouble());
        m_gameBar.setTitle("Wave " + m_gameState.waveProgress().value() + " / " + m_gameState.waveProgress().maximum());
        m_gameBar.setColor(BarColor.PURPLE);
        m_waveBar.setVisible(true);
        m_waveBar.setColor(BarColor.BLUE);
        m_waveBar.setTitle("Warmup - Waiting for players to get ready...");
        m_waveBar.setProgress(m_gameState.playerReadyProgress().toDouble());
        m_bombBar.setVisible(false);
        break;
      case Countdown:
        m_gameBar.setProgress(m_gameState.waveProgress().toDouble());
        m_gameBar.setTitle("Wave " + m_gameState.waveProgress().value() + " / " + m_gameState.waveProgress().maximum());
        m_gameBar.setColor(BarColor.PURPLE);
        m_waveBar.setVisible(true);
        m_waveBar.setColor(BarColor.YELLOW);
        m_waveBar.setTitle("Wave starting!");
        m_waveBar.setProgress(m_countdownProgress);
        m_bombBar.setVisible(false);
        break;
      case Playing:
        m_gameBar.setProgress(m_gameState.waveProgress().toDouble());
        m_gameBar.setTitle("Wave " + m_gameState.waveProgress().value() + " / " + m_gameState.waveProgress().maximum());
        m_gameBar.setColor(BarColor.PURPLE);
        if (m_gameState.mobProgress().maximum() > 0) {
          m_waveBar.setVisible(true);
          m_waveBar.setColor(BarColor.GREEN);
          m_waveBar.setTitle("Mobs remaining: " + (m_gameState.mobProgress().maximum() - m_gameState.mobProgress().value()));
          m_waveBar.setProgress(m_gameState.mobProgress().toDouble());
        } else {
          m_waveBar.setVisible(false);
        }
        if (m_gameState.fuseProgress().value() > 0) {
          m_bombBar.setVisible(true);
          m_bombBar.setProgress(1.0 - m_gameState.fuseProgress().toDouble());
        } else {
          m_bombBar.setVisible(false);
        }
        break;
      case GameOver:
        m_gameBar.setColor(BarColor.RED);
        m_gameBar.setProgress(1.0);
        m_gameBar.setTitle("Game Over!");
        m_waveBar.setVisible(true);
        m_waveBar.setColor(BarColor.BLUE);
        m_waveBar.setTitle("Returning to lobby...");
        m_waveBar.setProgress(m_countdownProgress);
        m_bombBar.setVisible(false);
        break;
    }
    for (Player p : m_respawnBars.keySet()) {
      Progress respawnProgress = m_gameState.playerRespawnProgress(p);
      if (respawnProgress.value() > 0) {
        m_respawnBars.get(p).setVisible(true);
        m_respawnBars.get(p).setProgress(respawnProgress.toDouble());
        m_respawnBars.get(p).setTitle("Respawning in " + respawnProgress.value());
      } else {
        m_respawnBars.get(p).setVisible(false);
      }
    }
  }
}
